/**
*
* CVE-Lite Source code
* (c) 2004 David Henderson & Dave Rowbotham
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Based on an idea introduced by Rob Conley ( estar ) in COMMON_ADDON
* GPDS Ver 2.0 based on GPDS 1.0 by David Henderson ( dh219 )
* CVE Libray coding by Dave Rowbotham.
* CVE-Lite adapted from CVE Library by D Henderson
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made.
*
*/

#define MAXPAYLOAD 50

#include <cstdio>
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include <fstream>
#include "cvelmessage.h"
#include "orbitersdk.h"

#ifndef _COMMON_MESH_EX_H_
#define _COMMON_MESH_EX_H_

/**
*
* container object for an orbiter mesh.
* To draw any mesh in any part of the ship we need just three
* properties to describe such a mesh, its Handle , its position
* and its direction/orientation.
*
*/

class COMMON_MESH
{
public:
	/**
	*
	*   COMMON_MESH()
	*   Constructor
	*/
	COMMON_MESH();

	/**
	*
	*   COMMON_MESH( const COMMON_MESH &cm );
	*   Copy Constructor
	*/
	COMMON_MESH( const COMMON_MESH &cm );

	~COMMON_MESH();
	/**
	*
	*   operator=( const COMMON_MESH &g )
	*   Assignment operator
	*/
	const COMMON_MESH & operator=( const COMMON_MESH &g );

	/**
	*   Get/Set the Mesh handle.
	*/
	MESHHANDLE MeshHandle();
	void       MeshHandle( MESHHANDLE m );

	/**
	*   Get/Set the Mesh position.
	*/
	VECTOR3    MeshPosition() { return mesh_position; };
	void       MeshPosition( const VECTOR3 &p ) { mesh_position = p; };

	/*
	*   Get/Set the Mesh name
	*/
	char        *cMeshName() { return (char*)mesh_name.c_str(); };
	std::string MeshName() { return mesh_name; };
	void        MeshName( const std::string &s ) { mesh_name = s; };
	void        MeshName( const char *s ) { mesh_name = s; };

	/*
	*   Get/Set the mesh index
	*/
	UINT	    MeshIndex();
	void        MeshIndex( UINT idx );

	/*
	*   Set the mesh visibility mode
	*/
	void        SetMeshVisibilityModeEx( VESSEL *parent, WORD mode );

	bool		MeshPayload() { return mesh_payload; };
	void		MeshPayload(bool v) { mesh_payload = v; };

protected:
//-----------------------------------------------------------------------------------
	MESHHANDLE	mesh_handle;		// plain orbiter mesh handle
	VECTOR3		mesh_position;		// position X,Y,Z
	std::string mesh_name;			// mesh filename sans .msh extension
	UINT		mesh_index;         // orbiter mesh index
	bool		mesh_payload;		// is this a payload mesh?
private:
};




//-----------------------------------------------------------------------------------

#endif



#ifndef __COMMON_ADDON_EX_H
#define __COMMON_ADDON_EX_H

// no debugging, all exceptions are silent
#define CVE_DEBUG_LEVEL0  0x00

// quiet debugging. exceptions and erros cause a logfile write
#define CVE_DEBUG_LEVEL1  0x01

// spammy debugging. exceptions and errors cause pop up spam and logfile write
#define CVE_DEBUG_LEVEL2  0x02

// uber spam no logfile. exceptions and errors only cause pop up spam. no logfile is written
#define CVE_DEBUG_LEVEL3  0x03


class _COMMON_VESSEL_EX
{
public:
	virtual void ParseCustomCVELString(char *key, char *string) {};

};
//-----------------------------------------------------------------------------------


/**
*
* extended vessel class
*/
class COMMON_VESSEL_EX: public VESSEL3 , public _COMMON_VESSEL_EX
{
public:

	COMMON_VESSEL_EX (OBJHANDLE hObj);
	~COMMON_VESSEL_EX();
	int SendMessage(const char* pszMessage, void* aMessage);

	/**
	*  Deploy the next payload in sequence
	*/
	VESSEL *DeployPayload();

	/*
	*  Deploy the next payload in seqence at VECTOR3 offset bias
	*/
	VESSEL *DeployPayload( const VECTOR3 &bias);
	void DeployAllPayloads();
	void DeployAllPayloads(const VECTOR3 &bias);

	void ParsePayloadString( char *payload_string );

	/**
	*
	*  Adds a mesh to this* CVE instance. It does not actually draw the mesh
	*  rather it adds it to an internally maintained list which is used by the
	*  actual "Draw" code.
	*/
	void AddMeshEx( const COMMON_MESH &m ) { self_mesh_list.push_back( m ) ; };

	/*
	* Clear all the meshes both visual and logical
	*/
	void ClearMeshesEx() ;

	/**
	* draw the ship and any payloads not deployed
	*/
	void Draw();

	/**
	* allow a user to offset all the drawings by a preset amount
	*/
	void Draw( const VECTOR3 &bias);

	double CalculateMass(VECTOR3 *payload_com); // calculates payload mass and adds passed offset
	double CalculateMass();	     // returns the total payload mass only

	/* write GPDS strings to a scenario file */

	void SaveDefaultStateEx( FILEHANDLE scn );


	/* Set custom payload departure velocity */

	void OverridePayloadVelocity(VECTOR3 &vel);


	/* cvelmessage handlers */

	void SendCentralMessage(VESSEL *vessel, char *key, char *message );
	bool CheckCentralMessage();
	bool CheckCentralMessage(char *in_key, char *in_message );

	/* set/get overall mesh visibility preferences */
	void MeshVisibilityPreference( DWORD meshvis_pref ) { meshvis = meshvis_pref; }
	DWORD MeshVisibilityPreference() { return meshvis; }

	VESSEL *JettisonComponent(char *name, char *classname, VECTOR3 &offset, VECTOR3 &rvel, VECTOR3 &vrot );

protected:

//-----------------------------------------------------------------------------------

	/* vector handling utilities */

	void normalise( VECTOR3 &v ) {
		v.x /= mod(v);
		v.y /= mod(v);
		v.z /= mod(v);
	}
	double mod( VECTOR3 &v ) { return sqrt( v.x*v.x + v.y*v.y + v.z*v.z ); }

	void COMMON_VESSEL_EX::ParseCVELString( char *payload_string );


	/* the PAYLOAD struct */

	typedef struct PAYLOAD {
		unsigned short type;   // 0 - PAYLOAD ; 1 - STAGE
		bool attached;
		signed short parent;
		char name[128];
		COMMON_MESH mesh;
		COMMON_MESH mesh_actual;
		VECTOR3 currentposition;
		char classname[128];
		double mass;
		double xrot;
		double fuel;
		VECTOR3 velocity;
	};

	PAYLOAD payload[MAXPAYLOAD];

	VECTOR3 _payload_bias ; // please don't use this, it breaks STAGEs!

private:

//-----------------------------------------------------------------------------------

	/* keep track of what staging level we're at */

	signed short currentparent;

	/*
	*
	*  Internal Variables
	*
	*  do not meddle here for bad things will happen if you do!
	*
	*/

	// meshes that are not part of the payload
	std::vector< COMMON_MESH> self_mesh_list;

	HINSTANCE hDLL;               // Handle to DLL

	DWORD meshvis;

	VECTOR3 payloadveloverride;
	bool payloadveloverride_active;

};

//-----------------------------------------------------------------------------------

extern "C" {
	typedef int (__cdecl *SENDCMSG)(VESSEL *vessel , char *key, char *message);
	typedef int (__cdecl *CHECKCMSG)(VESSEL *vessel , char *key, char *message);
}


#endif
