/**
*
* CVE-Lite Source code
* (c) 2004 David Henderson & Dave Rowbotham
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Based on an idea introduced by Rob Conley ( estar ) in COMMON_ADDON
* GPDS Ver 2.0 based on GPDS 1.0 by David Henderson ( dh219 )
* CVE Libray coding by Dave Rowbotham.
* CVE-Lite adapted from CVE Library by D Henderson
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/
#include "cvel.h"
HWND hRend;

/**
*
*   COMMON_VESSEL_EX(OBJHANDLE hObj)
*
*   Constructor
*   Nothing fancy here. Just sets the internal variables to
*   something sane.
*/
COMMON_VESSEL_EX::COMMON_VESSEL_EX(OBJHANDLE hObj)
: VESSEL3(hObj)
{
     
 try
 {
	ClearMeshesEx();

	int i;
	for( i = 0 ; i < MAXPAYLOAD ; i++ ) {
		payload[i].type = 0;
		payload[i].attached = false;
		payload[i].name[0] = 0;
		payload[i].parent = 0;
		payload[i].classname[0] = 0;
		payload[i].currentposition = _V(0,0,0);
		payload[i].fuel = 0.0;
		payload[i].xrot = 0.0;
		payload[i].mass = 0.0;
		payload[i].velocity = _V(0,0,0);
	}
	currentparent = -1;

	_payload_bias = _V(0,0,0);

	meshvis = MESHVIS_EXTERNAL;


	hDLL = LoadLibrary("Modules\\cvelmessage");

	
//	std::vector< COMMON_MESH>::iterator ii;

    
	payloadveloverride_active = false;
	payloadveloverride = _V(0,0,1);

 }
 catch(...)
 {

 }
}
//-----------------------------------------------------------------------------------


/**
*
*   ~COMMON_VESSEL_EX()
*
*   DeConstructor
*   Nothing fancy here. resets all variables by calling CloseCVE()
*/
COMMON_VESSEL_EX::~COMMON_VESSEL_EX()
{
	FreeLibrary(hDLL);	
}


void COMMON_VESSEL_EX::SaveDefaultStateEx(FILEHANDLE scn) {

	UINT i;
	OBJHANDLE hload;
	PAYLOAD p;
	char gpdsbuf[1024];

	hload = NULL;

	for( i = 0 ; i < MAXPAYLOAD && payload[i].attached ; i++ ) {

		p = payload[i];
		sprintf( gpdsbuf, "%s %s %s %.2lf %.2lf %.2lf %.2lf %.2lf %.2lf",
			p.name, p.mesh.cMeshName(), p.classname, p.mesh.MeshPosition().x, p.mesh.MeshPosition().y, p.mesh.MeshPosition().z, p.mass, p.xrot, p.fuel);

		if( p.type == 0 ) {
			oapiWriteScenario_string( scn, "PAYLOAD", gpdsbuf);
		}
		else if( p.type == 1 ) {
			oapiWriteScenario_string( scn, "STAGE", gpdsbuf);
		}					

		if( i < MAXPAYLOAD-1 && payload[i].parent != -1 ) {
			if( payload[i+1].parent < payload[i].parent || !payload[i+1].attached ) {
				oapiWriteScenario_string( scn, "ENDSTAGE", "");
			}
		}
	}

}

void COMMON_VESSEL_EX::OverridePayloadVelocity(VECTOR3 &vel) {

	payloadveloverride_active = true;
	payloadveloverride = vel;
}

VESSEL *COMMON_VESSEL_EX::JettisonComponent(char *name, char *classname, VECTOR3 &offset, VECTOR3 &rvel, VECTOR3 &vrot ) {

	VESSELSTATUS vs;
	VECTOR3 tmpvel;
	OBJHANDLE h;
	
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;

	Local2Rel (offset, vs.rpos);
	
	GlobalRot( rvel, tmpvel );
	vs.rvel += tmpvel;

	vs.vrot += vrot;

	h = oapiCreateVessel(name, classname, vs);

	return oapiGetVesselInterface(h);
}



void COMMON_VESSEL_EX::SendCentralMessage(VESSEL *vessel, char *key, char *message ) {
	SENDCMSG function;

	if (hDLL != NULL)
	{
		function = (SENDCMSG)GetProcAddress(hDLL, "SendCMessage");
		if (!function)
		{

		}
		else
		{
			function(vessel, key, message);
		}
	}
	else {

	}
	
}
bool COMMON_VESSEL_EX::CheckCentralMessage() {

	return CheckCentralMessage( NULL, NULL );
}

bool COMMON_VESSEL_EX::CheckCentralMessage(char *in_key, char *in_message ) {

	CHECKCMSG function;

	char key[256];
	char message[2048];
	int rc;
	bool data;

	rc = -1;
	data = false;
	if (hDLL != NULL)
	{
		function = (CHECKCMSG)GetProcAddress(hDLL, "CheckCMessage");
		if (!function)
		{
			/* Couldn't get address */
		}
		else
		{
		  // call the function
			do {
				rc = function((VESSEL*)this,key,message);
				if( rc != -1 ) {
					if( strlen( message ) ) {
						SendMessage(key,message);
						if( in_key != NULL && in_message != NULL ) {
							strncpy( in_key, key, 255 );
							strncpy( in_message, message, 2047 );
						}
						data = true;
					}
					else{
						/* zero-length message */
					}
				}
				else {
					/* no return */
				}
			} while( rc >= 0 );

		}
	}
	else {
		/* couldn't load library */
	}

	return data;
}
